@@ -35,13 +35,14 @@ class RecoverableException(Exception):
 def is_recoverable(exception):
     if exception is None or exception.code() is None:
         return False
+    if grpc.StatusCode.PERMISSION_DENIED == exception.code():
+        return check()
     return grpc.StatusCode.NOT_FOUND != exception.code() and \
            grpc.StatusCode.ALREADY_EXISTS != exception.code() and \
            grpc.StatusCode.UNKNOWN != exception.code() and \
            grpc.StatusCode.INVALID_ARGUMENT != exception.code() and \
            grpc.StatusCode.FAILED_PRECONDITION != exception.code() and \
-           grpc.StatusCode.UNAUTHENTICATED != exception.code() and \
-           grpc.StatusCode.PERMISSION_DENIED != exception.code()
+           grpc.StatusCode.UNAUTHENTICATED != exception.code()
 
 
 def connect():
@@ -83,7 +84,7 @@ def current_access_token_is_valid():
         access_token_dict = jwt.decode(access_token, options={"verify_signature":False})
         return access_token_dict['exp'] > time.time()
 
-
+# Refreshes the current token. Returns False if it was already valid
 def check():
     """Check whether there exist a working channel and a valid access token."""
     global bx_endpoint, bx_token
@@ -91,6 +92,8 @@ def check():
         connect(bx_endpoint, bx_token)
     if not current_access_token_is_valid():
         login()
+        return True
+    return False
 
 
 @retry(RecoverableException, delay=CALL_DELAY, backoff=CALL_BACKOFF, max_delay=CALL_MAX_DELAY, tries=MAX_TRIES)
@@ -538,8 +541,10 @@ class JobService:
         return make_call(job_pb2_grpc.JobServiceStub(channel).TagWorkflow, request)
 
     def Run(self, submit_request):
-        job = self.Submit(submit_request)
-        request_stream = self.StreamRequest(environment=submit_request.environment, job_id=job.job_id)
+        submit_response = self.Submit(submit_request)
+        if submit_response.WhichOneof("case") == "suggested_parameters":
+            raise Exception("Unsupported job parametrization. Suggested one is: " + str(getattr(submit_response,suggested_parameters)))
+        request_stream = self.StreamRequest(environment=submit_request.environment, job_id=submit_response.job_id)
         response = self.Stream(request_stream)
         for r in response:
             for job_event in r.event:
